"""Interactua con el datalake.

Modulos exportados por este paquete.

- `Datalake`: Clase principal para interactuar con el datalake.
"""

__author__ = """centraal.studio"""
__email__ = 'equipo@centraal.studio'
__version__ = '0.4.0'

from azure_datalake_utils.azure_datalake_utils import Datalake
